@@ -13,6 +13,7 @@ import hashlib
 import hmac
 from uuid import uuid4
 import six.moves.urllib as urllib
+from azure.core.credentials import AccessToken
 
 try:
     from urllib import quote, quote_plus, urlencode  # Py2
@@ -21,38 +22,11 @@ except Exception:
 
 import uamqp
 
-default_sas_expiry = 30
+
+default_sas_expiry = 3600
 
 
-class IoTHubAmqpClient:
-    def _generate_auth_token(self, uri, sas_name, sas_value):
-        sas = base64.b64decode(sas_value)
-        expiry = str(int(time.time() + default_sas_expiry))
-        string_to_sign = (uri + "\n" + expiry).encode("utf-8")
-        signed_hmac_sha256 = hmac.HMAC(sas, string_to_sign, hashlib.sha256)
-        signature = urllib.parse.quote(base64.b64encode(signed_hmac_sha256.digest()))
-        return "SharedAccessSignature sr={}&sig={}&se={}&skn={}".format(
-            uri, signature, expiry, sas_name
-        )
-
-    def _build_amqp_endpoint(self, hostname, shared_access_key_name, shared_access_key):
-        hub_name = hostname.split(".")[0]
-        endpoint = "{}@sas.root.{}".format(shared_access_key_name, hub_name)
-        endpoint = quote_plus(endpoint)
-        sas_token = self._generate_auth_token(
-            hostname, shared_access_key_name, shared_access_key + "="
-        )
-        endpoint = endpoint + ":{}@{}".format(quote_plus(sas_token), hostname)
-        return endpoint
-
-    def __init__(self, hostname, shared_access_key_name, shared_access_key):
-        self.endpoint = self._build_amqp_endpoint(
-            hostname, shared_access_key_name, shared_access_key
-        )
-        operation = "/messages/devicebound"
-        target = "amqps://" + self.endpoint + operation
-        self.amqp_client = uamqp.SendClient(target)
-
+class IoTHubAmqpClientBase:
     def disconnect_sync(self):
         """
         Disconnect the Amqp client.
@@ -90,8 +64,6 @@ class IoTHubAmqpClient:
                 msg_props.absolute_expiry_time = prop_value
             elif prop_key == "messageId":
                 msg_props.message_id = prop_value
-            elif prop_key == "userId":
-                msg_props.user_id = prop_value
             else:
                 app_properties[prop_key] = prop_value
 
@@ -102,3 +74,49 @@ class IoTHubAmqpClient:
         results = self.amqp_client.send_all_messages(close_on_done=False)
         if uamqp.constants.MessageState.SendFailed in results:
             raise Exception("C2D message send failure")
+
+
+class IoTHubAmqpClientSharedAccessKeyAuth(IoTHubAmqpClientBase):
+    def __init__(self, hostname, shared_access_key_name, shared_access_key):
+        def get_token():
+            expiry = int(time.time() + default_sas_expiry)
+            sas = base64.b64decode(shared_access_key)
+            string_to_sign = (hostname + "\n" + str(expiry)).encode("utf-8")
+            signed_hmac_sha256 = hmac.HMAC(sas, string_to_sign, hashlib.sha256)
+            signature = urllib.parse.quote(base64.b64encode(signed_hmac_sha256.digest()))
+            return AccessToken(
+                "SharedAccessSignature sr={}&sig={}&se={}&skn={}".format(
+                    hostname, signature, expiry, shared_access_key_name
+                ),
+                expiry,
+            )
+
+        auth = uamqp.authentication.JWTTokenAuth(
+            audience="https://" + hostname,
+            uri="https://" + hostname,
+            get_token=get_token,
+            token_type=b"servicebus.windows.net:sastoken",
+        )
+        auth.update_token()
+        self.amqp_client = uamqp.SendClient(
+            target="amqps://" + hostname + "/messages/devicebound", auth=auth
+        )
+
+
+class IoTHubAmqpClientTokenAuth(IoTHubAmqpClientBase):
+    def __init__(
+        self, hostname, token_credential, token_scope="https://iothubs.azure.net/.default"
+    ):
+        def get_token():
+            result = token_credential.get_token(token_scope)
+            return AccessToken("Bearer " + result.token, result.expires_on)
+
+        auth = uamqp.authentication.JWTTokenAuth(
+            audience=token_scope,
+            uri="https://" + hostname,
+            get_token=get_token,
+            token_type=b"bearer",
+        )
+        auth.update_token()
+        target = "amqps://" + hostname + "/messages/devicebound"
+        self.amqp_client = uamqp.SendClient(target=target, auth=auth)
