@@ -3,7 +3,7 @@
 # Licensed under the MIT License. See License.txt in the project root for
 # license information.
 # --------------------------------------------------------------------------
-from .iothub_amqp_client import IoTHubAmqpClient as iothub_amqp_client
+from . import iothub_amqp_client
 from .auth import ConnectionStringAuthentication, AzureIdentityCredentialAdapter
 from .protocol.iot_hub_gateway_service_ap_is import IotHubGatewayServiceAPIs as protocol_client
 from .protocol.models import (
@@ -56,9 +56,12 @@ class IoTHubRegistryManager(object):
     based on top of the auto generated IotHub REST APIs
     """
 
-    def __init__(self, connection_string=None, host=None, auth=None):
+    def __init__(self, connection_string=None, host=None, token_credential=None):
         """Initializer for a Registry Manager Service client.
 
+        Users should not call this directly. Rather, they should the from_connection_string()
+        or from_token_credential() factory methods.
+
         After a successful creation the class has been authenticated with IoTHub and
         it is ready to call the member APIs to communicate with IoTHub.
 
@@ -72,18 +75,24 @@ class IoTHubRegistryManager(object):
         :returns: Instance of the IoTHubRegistryManager object.
         :rtype: :class:`azure.iot.hub.IoTHubRegistryManager`
         """
+        self.amqp_svc_client = None
         if connection_string is not None:
-            self.auth = ConnectionStringAuthentication(connection_string)
-            self.protocol = protocol_client(self.auth, "https://" + self.auth["HostName"])
-            self.amqp_svc_client = iothub_amqp_client(
-                self.auth["HostName"],
-                self.auth["SharedAccessKeyName"],
-                self.auth["SharedAccessKey"],
+            conn_string_auth = ConnectionStringAuthentication(connection_string)
+            self.protocol = protocol_client(
+                conn_string_auth, "https://" + conn_string_auth["HostName"]
+            )
+            self.amqp_svc_client = iothub_amqp_client.IoTHubAmqpClientSharedAccessKeyAuth(
+                conn_string_auth["HostName"],
+                conn_string_auth["SharedAccessKeyName"],
+                conn_string_auth["SharedAccessKey"],
             )
         else:
-            self.auth = auth
-            self.protocol = protocol_client(self.auth, "https://" + host)
-            self.amqp_svc_client = None
+            self.protocol = protocol_client(
+                AzureIdentityCredentialAdapter(token_credential), "https://" + host
+            )
+            self.amqp_svc_client = iothub_amqp_client.IoTHubAmqpClientTokenAuth(
+                host, token_credential
+            )
 
     @classmethod
     def from_connection_string(cls, connection_string):
@@ -113,15 +122,13 @@ class IoTHubRegistryManager(object):
 
         :rtype: :class:`azure.iot.hub.IoTHubRegistryManager`
         """
-        host = url
-        auth = AzureIdentityCredentialAdapter(token_credential)
-        return cls(host=host, auth=auth)
+        return cls(host=url, token_credential=token_credential)
 
     def __del__(self):
         """
         Deinitializer for a Registry Manager Service client.
         """
-        if self.amqp_svc_client:
+        if self.amqp_svc_client is not None:
             self.amqp_svc_client.disconnect_sync()
 
     def create_device_with_sas(
@@ -923,6 +930,4 @@ class IoTHubRegistryManager(object):
 
         :raises: Exception if the Send command is not able to send the message
         """
-
-        if self.amqp_svc_client:
-            self.amqp_svc_client.send_message_to_device(device_id, message, properties)
+        self.amqp_svc_client.send_message_to_device(device_id, message, properties)
